'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.removeFile = exports.assembleFileChunks = exports.writeFileChunk = exports.writeFile = exports.readFile = exports.getFileSize = undefined;

var _fs = require('fs');

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var files = {};

var result = function result(status, data) {
  return {
    status: status,
    data: data || {}
  };
};

var exists = function exists(filename) {
  return !!files[filename];
};
var size = function size(filename) {
  return files[filename].size;
};
var read = function read(filename) {
  return files[filename].buffer || (0, _fs.readFileSync)(files[filename].filepath);
};
var remove = function remove(filename) {
  var file = files[filename];
  if (file.filepath) {
    (0, _fs.unlinkSync)(file.filepath);
  }
  delete files[filename];
  _log2.default.debug('File removed', filename);
};
var write = function write(filename, buffer, filesize, filepath) {
  files[filename] = {
    age: new Date().getTime(),
    buffer: buffer,
    filepath: filepath,
    size: buffer ? buffer.length : filesize
  };
  _log2.default.debug('File saved', files[filename]);
};

var getFileSize = exports.getFileSize = function getFileSize(filename) {
  if (exists(filename)) {
    var fileSize = size(filename);
    _log2.default.debug('Request size of', filename, 'is', fileSize);
    return result(200, { fileSize: fileSize });
  }
  _log2.default.debug('Request size of', filename, 'not found');
  return result(404);
};

var readFile = exports.readFile = function readFile(filename) {
  if (exists(filename)) {
    _log2.default.debug('Streaming', filename);
    return result(200, read(filename));
  }
  _log2.default.debug('Streaming', filename, 'not found');
  return result(404);
};

var writeFile = exports.writeFile = function writeFile(file) {
  _log2.default.debug('Storing', file.originalname);
  write(file.originalname, file.buffer, file.size, file.path);
  return result(200);
};
var writeFileChunk = exports.writeFileChunk = function writeFileChunk(filename, buffer, chunkNumber) {
  _log2.default.debug('Storing', filename, 'chunk', chunkNumber);
  write(filename + '.' + chunkNumber + '.chunk', buffer);
  return result(200);
};
var assembleFileChunks = exports.assembleFileChunks = function assembleFileChunks(filename, requestTotalSize) {
  _log2.default.debug('Assembling', filename, 'total size', requestTotalSize);
  var chunkNumber = 1;
  var totalSize = 0;
  while (true) {
    var chunkName = filename + '.' + chunkNumber + '.chunk';
    if (exists(chunkName)) {
      var fileSize = size(chunkName);
      _log2.default.debug('Testing', chunkName, 'with size', fileSize);
      chunkNumber += 1;
      totalSize += fileSize;
    } else {
      _log2.default.error('Testing', chunkName, 'not found');
      break;
    }
  }
  if (requestTotalSize !== totalSize) {
    _log2.default.error('Request total size', requestTotalSize, 'not equal to calculated total size', totalSize);
    return result(412);
  }
  _log2.default.debug('Request total size', requestTotalSize, 'equal to calculated total size', totalSize);
  var buffer = null;
  chunkNumber = 1;
  while (true) {
    var chunkNameX = filename + '.' + chunkNumber + '.chunk';
    if (!exists(chunkNameX)) {
      break;
    }
    buffer = buffer ? Buffer.concat([buffer, read(chunkNameX)]) : read(chunkNameX);
    remove(chunkNameX);
    chunkNumber += 1;
  }
  write(filename, buffer);
  return result(200);
};

var removeFile = exports.removeFile = function removeFile(filename) {
  if (exists(filename)) {
    _log2.default.debug('Removing file', filename);
    remove(filename);
    return result(200);
  }
  _log2.default.error('Removing', filename, 'not found');
  return result(404);
};